"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.inferSpecFromVueFile = void 0;
const vue_docgen_api_1 = require("vue-docgen-api");
const app_1 = require("@knapsack/app");
/**
 * @link https://vue-styleguidist.github.io/docs/Documenting.html
 */
async function inferSpecFromVueFile(filePath) {
    try {
        const componentDoc = await (0, vue_docgen_api_1.parse)(filePath);
        const { props = [], slots = [], events = [], methods = [] } = componentDoc;
        if (props.length === 0 &&
            slots.length === 0
        // events.length === 0 &&
        // methods.length === 0
        ) {
            return false;
        }
        const spec = {
            props: {
                $schema: 'http://json-schema.org/draft-07/schema',
                type: 'object',
                required: [],
                properties: {},
            },
            slots: {
            // children: {
            //   title: 'children',
            // },
            },
        };
        props.forEach((prop) => {
            const { name: propName, defaultValue, description, type, required, values, } = prop;
            switch (type === null || type === void 0 ? void 0 : type.name) {
                case 'string':
                    if (values) {
                        spec.props.properties[propName] = {
                            type: 'string',
                            enum: values,
                        };
                    }
                    else {
                        spec.props.properties[propName] = {
                            type: 'string',
                        };
                    }
                    if (defaultValue && 'value' in defaultValue) {
                        spec.props.properties[propName].default =
                            defaultValue.value.replace(/'/g, ''); // value can be `{ value: "'left'" }`
                    }
                    break;
                case 'number':
                    spec.props.properties[propName] = {
                        type: 'number',
                    };
                    if (defaultValue && 'value' in defaultValue) {
                        spec.props.properties[propName].default = parseInt(defaultValue.value, 10); // value can be `{ value: "8" }`
                    }
                    break;
                case 'boolean':
                    spec.props.properties[propName] = {
                        type: 'boolean',
                    };
                    if (defaultValue && 'value' in defaultValue) {
                        spec.props.properties[propName].default =
                            // eslint-disable-next-line eqeqeq
                            defaultValue.value == 'true'; // value can be `{ value: 'true' }`
                    }
                    break;
                case 'func':
                    spec.props.properties[propName] = {
                        typeof: 'function',
                        tsType: '() => void',
                    };
                    break;
            }
            // assigning info that all would have
            if (spec.props.properties[propName]) {
                if (required)
                    spec.props.required.push(propName);
                if (description && !spec.props.properties[propName].description) {
                    spec.props.properties[propName].description = description;
                }
                if (defaultValue &&
                    'value' in defaultValue &&
                    !('default' in spec.props.properties[propName])) {
                    spec.props.properties[propName].default = defaultValue.value;
                }
            }
        });
        slots.forEach((slot) => {
            const { name, description = '' } = slot;
            spec.slots[name] = {
                title: name,
                description,
            };
        });
        const extraDocs = [];
        if (events.length > 0) {
            extraDocs.push({
                id: 'events',
                title: 'Events',
                description: `These custom \`events\` can be listened to - [docs](https://vuejs.org/v2/guide/components-custom-events.html)`,
                tableRows: events.map((event) => {
                    const { name, description } = event;
                    return {
                        Name: `\`${name}\``,
                        Description: description,
                    };
                }),
            });
        }
        if (extraDocs.length > 0) {
            spec.extraDocs = extraDocs;
        }
        // log.inspect({ componentDoc, spec }, 'infer vue spec');
        return spec;
    }
    catch (e) {
        console.log(e);
        app_1.log.error(`Error while trying to inferSpec of Vue template: ${filePath} - ${e.message}`);
        return false;
    }
}
exports.inferSpecFromVueFile = inferSpecFromVueFile;
//# sourceMappingURL=infer-spec.js.map