"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KnapsackVueRenderer = void 0;
const renderer_webpack_base_1 = require("@knapsack/renderer-webpack-base");
const app_1 = require("@knapsack/app");
const vue_loader_1 = require("vue-loader");
const utils_1 = require("@knapsack/utils");
const file_utils_1 = require("@knapsack/file-utils");
const path_1 = require("path");
const utils_2 = require("./utils");
const infer_spec_1 = require("./infer-spec");
const { pkg } = (0, file_utils_1.findUpPkgJson)(__dirname);
const gitRoot = (0, file_utils_1.findGitRoot)();
// eslint-disable-next-line @typescript-eslint/no-non-null-assertion
app_1.log.setupUpdateNotifier({ ...pkg, name: pkg.name, version: pkg.version });
class KnapsackVueRenderer extends renderer_webpack_base_1.KnapsackRendererWebpackBase {
    constructor({ webpackConfig, skipWebpackAlter = false, } = {}) {
        super({
            id: 'vue',
            language: 'vue',
            extension: '.vue',
            webpackConfig,
            extraScripts: [(0, path_1.join)(__dirname, './renderer-vue-client')],
        });
        this.getMeta = () => ({
            id: this.id,
            title: 'Vue',
            aliasUse: 'off',
            enableDataDemos: true,
            enableTemplateDemos: true,
            hasSlotsSupport: true,
            version: pkg.version,
            hasInferSpecSupport: true,
            hasTemplateSuggestionsSupport: true,
        });
        this.inferSpec = async ({ template, templatePath, }) => {
            const spec = await (0, infer_spec_1.inferSpecFromVueFile)(templatePath);
            return spec;
        };
        this.language = 'vue';
        this.skipWebpackAlter = skipWebpackAlter;
    }
    createWebpackConfig() {
        const config = super.createWebpackConfig();
        if (this.skipWebpackAlter)
            return config;
        // https://vue-loader.vuejs.org/options.html
        const compilerOptions = {
            compilerOptions: {
                compatConfig: {
                    MODE: 2,
                },
            },
        };
        // Parses vue files and hands script to js/ts loaders, style to css/etc loaders
        config.module.rules.push({
            test: /\.vue$/,
            loader: require.resolve('vue-loader'),
            options: compilerOptions,
        });
        // vue-style-loader provided by VueLoaderPlugin
        // If site needs LESS/Sass, override at knapsack.config.js within project
        config.module.rules.push({
            test: /\.css$/,
            use: [require.resolve('vue-style-loader'), require.resolve('css-loader')],
        });
        config.module.rules.push({
            test: [/\.js$/, /\.jsx$/, /\.ts$/, /\.tsx$/],
            exclude: /node_modules/,
            loader: require.resolve('babel-loader'),
            options: this.basicWebpackBabelConfig,
        });
        config.plugins.push(new vue_loader_1.VueLoaderPlugin());
        return config;
    }
    changeCase(str) {
        const templateName = (0, utils_1.paramCase)(str);
        if (templateName.split('-').length === 1) {
            // vue templates (and web components) MUST contain at least one dash
            return `vue-${templateName}`;
        }
        return templateName;
    }
    async render(opt) {
        const { usage, imports: nonUniqueImports } = await this.getUsageAndImports(opt);
        const { imports, declaredVars, nameCollisions, isDeclaredVarsUnique } = this.makeKsJsImportsUnique({
            imports: nonUniqueImports,
        });
        if (!isDeclaredVarsUnique) {
            throw new Error(`imports are not unique: ${nameCollisions.join(', ')}`);
        }
        let usageDocs = usage;
        const code = `
    ${this.createHtmlTagsForAssetPaths({
            assets: this.getWebpackAssetPaths(),
            // If we don't have `<script async>` tags and we put this below the `div#render-root` then renderering happens but the iFrame is too narrow. Not sure why using a `setTimeout` around `new Vue()` or putting this below `div#render-root` doesn't work, but this does
            scriptTagsAreAsync: true,
        })}
    <div id="render-root" class="knapsack-pattern-direct-parent" data-dev-note="Knapsack Vue Template Wrapper" style="display: none;">${usage}</div>
    `;
        if ((opt === null || opt === void 0 ? void 0 : opt.demo) && KnapsackVueRenderer.isDataDemo(opt.demo)) {
            usageDocs = app_1.KnapsackRendererBase.formatCode({
                language: 'vue',
                code: `
<template>
  ${usage}
</template>

<script>
export default {
  components: {
    ${imports
                    .map(({ importInfo }) => `'${importInfo.name}': () => import('${importInfo.path}'),`)
                    .join('\n')}
  }
}
</script>
        `,
            });
        }
        if ((opt === null || opt === void 0 ? void 0 : opt.demo) && KnapsackVueRenderer.isTemplateDemo(opt.demo)) {
            const pathInfo = opt.patternManifest.getTemplatePath({
                patternId: opt.pattern.id,
                templateId: opt.template.id,
                demoId: opt.demo.id,
            });
            const demoContents = await (0, file_utils_1.readFile)(pathInfo.absolutePath);
            usageDocs = app_1.KnapsackRendererBase.formatCode({
                language: 'vue',
                code: demoContents,
            });
        }
        // log.inspect({ demoType: opt.demo.type, code, imports });
        return {
            ok: true,
            templateLanguage: 'html',
            usage: usageDocs,
            html: app_1.KnapsackRendererBase.formatCode({
                code,
                language: 'html',
            }),
        };
    }
    async getUsage(opt) {
        const { usage } = await this.getUsageAndImports(opt);
        return usage;
    }
    async getUsageAndImports(opt) {
        var _a, _b, _c, _d;
        const { pattern, template, demo, 
        /**
         * The Patterns class
         */
        patternManifest, } = opt;
        switch (demo === null || demo === void 0 ? void 0 : demo.type) {
            case 'template': {
                const pathInfo = opt.patternManifest.getTemplatePath({
                    patternId: pattern.id,
                    templateId: template.id,
                    demoId: demo.id,
                });
                const importInfo = this.getJsImport({
                    patternId: pattern.id,
                    templateId: template.id,
                    demoId: demo.id,
                });
                const templateName = importInfo.importInfo.name;
                return {
                    usage: `<${templateName} />`,
                    imports: [importInfo],
                };
            }
            case 'data':
            default: {
                const props = (_b = (_a = demo === null || demo === void 0 ? void 0 : demo.data) === null || _a === void 0 ? void 0 : _a.props) !== null && _b !== void 0 ? _b : {};
                const slots = (_d = (_c = demo === null || demo === void 0 ? void 0 : demo.data) === null || _c === void 0 ? void 0 : _c.slots) !== null && _d !== void 0 ? _d : {};
                const importInfo = this.getJsImport({
                    patternId: pattern.id,
                    templateId: template.id,
                });
                if (!importInfo) {
                    throw new Error(`Could not find import for pattern-template: ${pattern.id}-${template.id}`);
                }
                const { type, name: templateName } = importInfo.importInfo;
                const importInfos = [importInfo];
                const children = [];
                await Promise.all(Object.keys(slots).map(async (slotName) => {
                    const slottedTemplates = slots[slotName];
                    await Promise.all(slottedTemplates.map(async (slottedTemplate) => {
                        if (app_1.KnapsackRendererBase.isSlottedText(slottedTemplate)) {
                            return slottedTemplate.text;
                        }
                        if (app_1.KnapsackRendererBase.isSlottedTemplateReference(slottedTemplate)) {
                            throw new Error(`Cannot handle slotted template references in ${JSON.stringify({
                                patternId: pattern.id,
                                templateId: template.id,
                                demoId: demo.id,
                                slottedTemplate,
                            })}`);
                        }
                        if (!slottedTemplate.patternId ||
                            !slottedTemplate.templateId ||
                            !slottedTemplate.demoId) {
                            return '';
                        }
                        const thisPattern = patternManifest.getPattern(slottedTemplate.patternId);
                        const thisTemplate = thisPattern.templates.find((t) => t.id === slottedTemplate.templateId);
                        if (!thisTemplate) {
                            throw new Error(`Cannot find template id "${slottedTemplate.templateId}" in "${slottedTemplate.patternId}"`);
                        }
                        const { usage, imports } = await this.getUsageAndImports({
                            pattern: thisPattern,
                            template: thisTemplate,
                            patternManifest,
                            demo: slottedTemplate.demo ||
                                this.patterns.demosById[slottedTemplate.demoId],
                        });
                        imports.forEach((i) => importInfos.push(i));
                        return usage;
                    })).then((usages) => {
                        if (slotName === 'default') {
                            children.push(usages.join('\n'));
                        }
                        else {
                            children.push(`<template v-slot:${slotName}> ${usages.join('\n')} </template>`);
                        }
                    });
                }));
                const pathInfo = opt.patternManifest.getTemplatePath({
                    patternId: pattern.id,
                    templateId: template.id,
                });
                const usage = await (0, utils_2.getUsage)({
                    templateName,
                    props,
                    children: children.join('\n'),
                });
                return {
                    usage,
                    imports: importInfos,
                };
            }
        }
    }
    async getTemplateSuggestions() {
        const { unusedPaths } = await this.getUnusedTemplatePaths({
            globPaths: [(0, path_1.join)(gitRoot, '**/*.vue')],
        });
        const suggestions = await Promise.all(unusedPaths.map(async (path) => {
            const pkgInfo = await (0, file_utils_1.findUpPkgJsonAsync)(path);
            if (!pkgInfo.exists) {
                return {
                    path,
                };
            }
            const { name } = pkgInfo.pkg;
            const subPath = (0, path_1.relative)(pkgInfo.pkgDir, path);
            const pkgSubPath = (0, path_1.join)(name, subPath);
            try {
                // try out the resolved path
                // if the "exports" field is used in the package.json, and we have problems with it, then this will throw and we'll just use the original absolute path
                const resolvedPath = require.resolve(pkgSubPath);
                if (resolvedPath === path) {
                    return {
                        path: pkgSubPath,
                    };
                }
                return {
                    path,
                };
            }
            catch (e) {
                return {
                    path,
                };
            }
        }));
        return { suggestions };
    }
}
exports.KnapsackVueRenderer = KnapsackVueRenderer;
//# sourceMappingURL=renderer-vue.js.map